/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PVM_CUSTOM_HH__
#define __PVM_CUSTOM_HH__

/** Base class for classes with non-standard packing and unpacking.

    This class is the base class for all data, you want to transmit,
    for which there are no standard PvmRegister()-Calls (see
    \Ref{PvmStruct}). For all types, that can be registered via
    PvmRegister()-Calls, there are functions called PvmPack( const
    Type & ) and PvmUnpack ( type ), which can be used in the Pack()
    and UnPack()-methods of PvmCustom. You can also use this
    mechanism, if you want to transmit data "compressed". For example
    you could have a huge array, that is mostly used with little data:

    \begin{verbatim}
      struct MyHugeArray : public PvmCustom
      {
        int Size;
        int Huge[100000];
        void Pack () const
          {
            PvmPack (Size);
            for (int i = 0; i < Size; ++i)
              PvmPack (Huge[i]);
          }
        void UnPack ()
          {
            PvmUnpack (Size);
            for (int i = 0; i < Size; ++i)
              PvmUnpack (Huge[i]);
          }
      };

      struct Test : public PvmStruct
      {
        PvmSetStructId (43); 
        PvmRegistration ()
        {
          PvmRegister (Data);
        }
        MyHugeArray Data;
      };

      int 
      main ()
      {
        Test A;
        A.Data.Size = 3;
        A.Data.Huge[ 0 ] = 4;
        A.Data.Huge[ 1 ] = 8;
        A.Data.Huge[ 2 ] = 16;
        A.Send (Pvm ().I ().Parent ()); // Only 4 ints are sent, not 100001 !
      }
    \end{verbatim} */

class PvmCustom
{
public:
  /** must be overridden with a function, that packs all relevant data
      of the derived class. */
  virtual void Pack () const = 0;

  /** must be overridden with a function, that unpacks all relevant
      data of the derived class. */
  virtual void UnPack () = 0;
  virtual ~PvmCustom ();
};

#endif /* __PVM_CUSTOM_HH__ */
