/* 
 * PVM++ - A C++ library for easy use of PVM
 * Copyright (C) 1997-2001 Sebastian Wilhelmi; University of Karlsruhe
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PVM_STRUCT_HH__
#define __PVM_STRUCT_HH__

#include <pvm++/types.hh>

template< class Type > Type * PvmGetNewInstance (const Type &);

#define PvmSetStructId(n)						\
virtual PvmStructId GetStructId () const { return n; } 			\
virtual PvmStruct* GetNewInstance() const				\
{									\
  return PvmGetNewInstance (*this);					\
}
#define PvmRegistration() virtual void PackOrUnPack ()

/** Base class for all classes to be transmitted by PVM++.

    Every struct (or class, for that matter) that should be sent
    between tasks must be publically derived from PvmStruct. And it
    has to register all its members, that should be transmitted, to
    PVM++. Finally it has to provide a tag, which is used internally
    in PVM++ to determine the type of a message upon arrival, just
    like in PVM. {\bf It is very important not to use one number for
    different classes and to use the same number for the same type in
    the different communicating programs!} For example a definition
    could look like this:

    \begin{verbatim}
      struct Test : public PvmStruct
      {
        PvmSetStructId (42); // Setting the Tag for this struct. 
                              // Don't use twice!!
        PvmRegistration ()
        {
          PvmRegister (Data);
          PvmRegister (Character);
          PvmRegister (DoubleArray, 187);
          PvmRegister (Host);
          PvmRegister (IntSet);
        }
        int Data;
        char Character;
        double DoubleArray[187];
        PvmHost Host;
        set< int > IntSet;
      };
    \end{verbatim}

    As you can see, STL types can be used and will be transmitted
    correctly, if they are registered and if the template arguments
    (i.e. int in set< int >) can be registered. The
    following list shows all types, that can be registered (and
    therefore transmitted):

    \begin{itemize}
    \item bool.
    \item char.
    \item unsigned char.
    \item short int.
    \item int.
    \item float.
    \item double.
    \item long int.
    \item unsigned short.
    \item unsigned int.
    \item unsigned long int.
    \item string.
    \item float_complex.
    \item double_complex.
    \item \Ref{PvmHost}.
    \item \Ref{PvmHostSet}.
    \item \Ref{PvmTask}.
    \item \Ref{PvmTaskSet}.
    \item pair< First, Second >, if First and Second can be registered.
    \item vector< Key >, if Key can be registered.
    \item list< Key >, if Key can be registered.
    \item stack< Key >, if Key can be registered.
    \item deque< Key >, if Key can be registered.
    \item set< Key >, if Key can be registered.
    \item multiset< Key >, if Key can be registered (not for gcc 2.7.2).
    \item map< Key >, if Key can be registered.
    \item multimap< Key >, if Key can be registered (not for gcc 2.7.2).
    \item Classes derived from \Ref{PvmStruct}.
    \item Classes derived from \Ref{PvmCustom}.
    \end{itemize}

    Arrays for all of the above types (except \Ref{PvmStruct} and
    \Ref{PvmCustom}, where it isn't possible due to inheritance) are
    supported as well. The corresponding syntax is PvmRegister (type
    *PointerToArray, int Size). */

class PvmStruct
{
public:
  static bool CurrentlyPacking;
  PvmStruct ();
  PvmStruct (const PvmStruct &What);
  PvmStruct & operator = (const PvmStruct &What);
  virtual ~PvmStruct ();
  virtual PvmStruct *GetNewInstance () const = 0;
  virtual PvmStructId GetStructId () const = 0;

  /** sends the message to task To. */
  void Send (PvmTask To) const;
  
  /** sends the message to all tasks in the set To. */
  void Send (const PvmTaskSet &To) const;

  /** receives a message of the given type and returns the id of the
      sender in From. The parameter can be omitted, if you're
      not interested in who was sending. The call blocks until a
      message is received. */
  void Receive (PvmTask &From = IgnoreTask);

  /** receives a message of the given type, but only from one of the
      tasks given by FromSet, and returns the id of the sender in
      From. This parameter can be omitted. The call blocks until a
      message is received. */
  void ReceiveFrom (const PvmTaskSet &FromSet, PvmTask &From = IgnoreTask);

  /** receives a message of the given type, but only from the task
      From. The call blocks until a message is received. */
  void ReceiveFrom (PvmTask From);

  /** receives a message of the given type and returns the id of the
      sender in From. This parameter can be omitted. The parameter
      Time specifies the maximal blocking time in microseconds. If
      after this time no message is received, than the function
      returns false, otherwise it returns true and the remaining time
      in Time immediately after receiving the message. */
  bool TimedReceive (unsigned long int &Time, PvmTask &From = IgnoreTask);

  /** receives a message of the given type, but only from one of the
      tasks given by FromSet, and returns the id of the sender in
      From. This parameter can be omitted. The parameter Time
      specifies the maximal blocking time in microseconds. If after
      this time no message is received, than the function returns
      false, otherwise it returns true and the remaining time in Time
      immediately after receiving the message. */
  bool TimedReceiveFrom (const PvmTaskSet &FromSet,
			 unsigned long int &Time,
			 PvmTask &From = IgnoreTask);

  /** receives a message of the given type, but only from the task
      From. The parameter Time specifies the maximal blocking time in
      microseconds. If after this time no message is received, than
      the function returns false, otherwise it returns true and the
      remaining time in Time immediately after receiving the
      message. */
  bool TimedReceiveFrom (PvmTask From, unsigned long int &Time);

  /** sets the new action on receive for the current message type to
      What.  The old action is returned for later use with this
      function call. */
  PvmReceiveAction ReceiveAction (const PvmReceiveAction &What);

  /** installs the handler Func as a message handler for all messages
      of the current message type. Such messages can't be received
      after a call to that function, as every time, such a message
      arrives, the installed handler will be called. The old action is
      returned for later use with the method ReceiveAction().  The
      parameter Func is of type void (*PvmHandlerFunc)( const class
      \Ref{PvmStruct}&, const class \Ref{PvmTask}& ). It gets the
      received message and the sender of it as parameters. A message
      handler might change global variables and send messages to other
      tasks, but it should {\bf never} try to receive messages!
      Message handlers are not called asynchronous, i.e. they are
      called, whenever the program calls a Send() or
      Receive*()-function or the Update()-function of
      \Ref{PvmClass}. */
  PvmReceiveAction InstallHandler (PvmHandlerFunc Func);

  /** installs the "unpack on receive" action, i.e. every subsequent
      arriving message of this type will automatically be unpacked
      into this instance. If this instance is deleted, NormalReceive
      is enabled again. The old action is returned for later use with
      the method ReceiveAction(). */
  PvmReceiveAction AutomaticUnPack ();

  /** installs the "swallow on receive" action, i.e. every subsequent
      arriving message of this type will be ignored. The old action is
      returned for later use with the method ReceiveAction(). */
  PvmReceiveAction SwallowOnReceive ();

  /** switches back to normal mode of operation, i.e. you have to
      receive messages yourself. The old action is returned for later
      use with the method ReceiveAction(). */
  PvmReceiveAction NormalReceive ();

protected:
  virtual void PackOrUnPack () = 0;
private:
  static PvmTask IgnoreTask;
  int StructIdForAutomaticUnpack;
  friend class PvmInternal;
  friend class PvmHandlerTableType;
  friend void PvmPack (const PvmStruct &What);
  friend void PvmUnPack (PvmStruct &What);
  void Pack () const;
  void UnPack ();
};

#endif /* __PVM_STRUCT_HH__ */
